<?php
if ( ! defined( 'MYCRED_DEBUG_BANKING_VER' ) ) exit;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'get_banking_addon_settings_mycred' ) ) :
	function get_banking_addon_settings_mycred( $point_type = 'mycred_default' ) {

		$option_id = 'mycred_pref_bank';
		if ( $point_type != 'mycred_default' )
			$option_id .= '_' . sanitize_key( $point_type );

		$addon_settings = mycred_get_option( $option_id, false );
		if ( $addon_settings === false )
			$addon_settings = array(
				'active'        => array(),
				'services'      => array(),
				'service_prefs' => array()
			);

		return $addon_settings;

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'mycred_date_to_gmt_time' ) ) :
	function mycred_date_to_gmt_time( $string = '' ) {

		return strtotime( get_gmt_from_date( $string ) );

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'mycred_gmt_time_to_local_time' ) ) :
	function mycred_gmt_time_to_local_time( $string = '' ) {

		return strtotime( get_date_from_gmt( date( 'Y-m-d H:i:s', $string ), 'Y-m-d H:i:s' ) );

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'mycred_have_log_entries' ) ) :
	function mycred_have_log_entries( $point_type = 'mycred_default' ) {

		global $wpdb;

		$mycred = mycred( $point_type );

		$check = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM {$mycred->log_table} WHERE ctype = %s", $point_type ) );
		if ( $check === NULL || $check == 0 )
			return false;

		return $check;

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'mycred_have_compounded_interest' ) ) :
	function mycred_have_compounded_interest( $point_type = 'mycred_default' ) {

		global $wpdb;

		$has_compound = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT( DISTINCT user_id ) FROM {$wpdb->usermeta} WHERE meta_key = %s;", $point_type . '_comp' ) );
		if ( $has_compound === NULL || $has_compound == 0 )
			return false;

		return $has_compound;

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'get_qualified_users_banking' ) ) :
	function get_qualified_users_banking( $point_type, $id_list = '', $exclude_roles = array(), $minimum = 0, $return_count = true ) {

		global $wpdb;

		$mycred      = mycred( $point_type );
		$exclude_ids = array();
		$minimum     = abs( $minimum );

		if ( strlen( $id_list ) > 0 ) {

			foreach ( explode( ',', $id_list ) as $user_id ) {
				$user_id       = absint( preg_replace( "/[^0-9]/", "", $user_id ) );
				if ( $user_id === 0 ) continue;
				$exclude_ids[] = $user_id;
			}

		}

		if ( ! empty( $exclude_roles ) ) {

			$args             = array();
			$args['role__in'] = $exclude_roles;
			$args['fields']   = 'ID';
			$user_query       = new WP_User_Query( $args );
			$user_ids         = $user_query->get_results();

			if ( ! empty( $user_ids ) ) {
				foreach ( $user_ids as $user_id ) {
					if ( in_array( $user_id, $exclude_ids ) ) continue;
					$exclude_ids[] = absint( $user_id );
				}
			}

		}

		if ( $mycred->exclude['list'] != '' ) {

			foreach ( explode( ',', $mycred->exclude['list'] ) as $user_id ) {
				$user_id       = absint( preg_replace( "/[^0-9]/", "", $user_id ) );
				if ( $user_id === 0 || in_array( $user_id, $exclude_ids ) ) continue;
				$exclude_ids[] = $user_id;
			}

		}

		if ( $minimum != '' && $mycred->number( $minimum ) > $mycred->zero() ) {

			$format = '%d';
			if ( $mycred->format['decimals'] > 0 )
				$format = '%f';

			$user_ids = $wpdb->get_col( $wpdb->prepare( "SELECT user_id FROM {$wpdb->usermeta} WHERE meta_key = %s AND meta_value < {$format};", $point_type, $minimum ) );
			if ( ! empty( $user_ids ) ) {
				foreach ( $user_ids as $user_id ) {
					if ( in_array( $user_id, $exclude_ids ) ) continue;
					$exclude_ids[] = absint( $user_id );
				}
			}

		}

		$exclude_list = '';
		if ( ! empty( $exclude_ids ) )
			$exclude_list = " AND user_id NOT IN (" . implode( ', ', $exclude_ids ) . ")";

		if ( $return_count ) {

			$count        = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT( DISTINCT user_id ) FROM {$mycred->log_table} WHERE ctype = %s {$exclude_list};", $point_type ) );
			if ( $count === NULL ) $count = 0;

			return $count;

		}

		return $exclude_ids;

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'get_users_with_mycred_balance' ) ) :
	function get_users_with_mycred_balance( $amount = 0, $point_type = 'mycred_default', $mycred = NULL ) {

		if ( ! is_object( $mycred ) )
			$mycred = mycred( $point_type );

		global $wpdb;

		$format = '%d';
		if ( $mycred->format['decimals'] > 0 )
			$format = '%f';

		$results = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT( DISTINCT user_id ) FROM {$wpdb->usermeta} WHERE meta_key = %s AND meta_value > {$format};", $point_type, $amount ) );
		if ( $results === NULL ) $results = 0;

		$results = absint( $results );
		if ( $results > 0 ) {

			$settings = get_banking_addon_settings_mycred( $point_type );
			$enabled  = ( ( in_array( 'central', $settings['active'] ) ) ? true : false );

			if ( $enabled ) {

				$settings = $settings['service_prefs']['central'];
				if ( strlen( $settings['bank_id'] ) > 0 )
					$results --;

			}

		}

		return $results;

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'get_bank_cron_schedule_form' ) ) :
	function get_bank_cron_schedule_form( $point_type = 'mycred_default', $service = '', $item = '' ) {

		$description = '';
		if ( $service == 'interest' ) {
			$description = 'When should the compounded interest be paid out the first time? The Cron job will re-schedule itself based on your setup for the next payout after this one.';
			if ( $item == 'compound' )
				$description = 'When should the compounding start? The schedule will then run on the same time each day until the service is disabled.';
		}

		ob_start();

?>
<div class="padded-form-body">
	<input type="hidden" name="action" id="cron-schedule-field-action" value="reschedule" />
	<input type="hidden" name="point_type" id="cron-schedule-field-point-type" value="<?php echo esc_attr( $point_type ); ?>" />
	<input type="hidden" name="service" id="cron-schedule-field-service" value="<?php echo esc_attr( $service ); ?>" />
	<input type="hidden" name="item" id="cron-schedule-field-item" value="<?php echo esc_attr( $item ); ?>" />
	<p><?php echo $description; ?></p>
	<table class="schedule-table">
		<thead>
			<tr>
				<th class="col-year">Year</th>
				<th class="col-month">Month</th>
				<th class="col-day">Day</th>
				<th class="col-sep">&nbsp;</th>
				<th class="col-hour">Hour</th>
				<th class="col-sep">&nbsp;</th>
				<th class="col-minute">Minute</th>
			</tr>
		</thead>
		<tbody>
			<tr>
				<td class="col-year">
					<input type="text" name="time[year]" class="large-input" placeholder="YYYY" maxlength="4" size="8" value="" />
				</td>
				<td class="col-month">
					<input type="text" name="time[month]" class="large-input" placeholder="MM" maxlength="2" size="4" value="" />
				</td>
				<td class="col-day">
					<input type="text" name="time[day]" class="large-input" placeholder="DD" maxlength="2" size="4" value="" />
				</td>
				<td class="col-sep">@</td>
				<td class="col-hour">
					<input type="text" name="time[hour]" class="large-input" placeholder="00" maxlength="2" size="4" value="" />
				</td>
				<td class="col-sep">:</td>
				<td class="col-minute">
					<input type="text" name="time[minute]" class="large-input" placeholder="00" maxlength="2" size="4" value="" />
				</td>
		</tbody>
	</table>
	<p><em>The hours must be set using 24 hours and not 12 hours. Keywords like AM or PM is not supported.</em></p>
	<p>
		<input type="submit" class="button button-primary" value="Schedule" id="schedule-form-submit-button" />
	</p>
</div>
<?php

		$content = ob_get_contents();
		ob_end_clean();

		return $content;

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'get_bank_task_run_form' ) ) :
	function get_bank_task_run_form( $point_type = 'mycred_default', $service = '', $item = '', $rerun = false ) {

		global $wpdb;

		$mycred   = mycred( $point_type );
		$settings = get_banking_addon_settings_mycred( $point_type );
		$settings = $settings['service_prefs'][ $service ];

		$first_entry = $wpdb->get_var( $wpdb->prepare( "SELECT time FROM {$mycred->log_table} WHERE ctype = %s ORDER BY id ASC LIMIT 1;", $point_type ) );

		ob_start();

?>
<div class="padded-form-body">
	<input type="hidden" name="action" id="run-task-field-action" value="run" />
	<input type="hidden" name="point_type" id="run-task-field-point-type" value="<?php echo esc_attr( $point_type ); ?>" />
	<input type="hidden" name="service" id="run-task-field-service" value="<?php echo esc_attr( $service ); ?>" />
	<input type="hidden" name="item" id="run-task-field-item" value="<?php echo esc_attr( $item ); ?>" />

	<div id="task-warning"<?php if ( $rerun === true ) echo ' style="display: none;"'; ?>>
		<p>Using your log, this tool can calculate and payout interest on your users balances, or on the amount of <?php echo $mycred->plural( false ); ?> they accumulated each day. Remember that changes to your users balances that were made without a log entry will not be taken into account when calculating interest. Based on the number of qualified users and the number of log entries, this process can take some time to complete. It is highly recommended that you run this task when your website has low traffic as the server will be busy and slow while the task runs.</p>
		<p><button type="button" class="button button-primary slide-content" data-up="task-warning" data-down="task-<?php echo esc_attr( $point_type ); ?>-<?php echo esc_attr( $service ); ?>">I Understand - Continue</button></p>
	</div>
	<div id="task-<?php echo esc_attr( $point_type ); ?>-<?php echo esc_attr( $service ); ?>" style="<?php if ( $rerun !== true ) echo 'display: none;'; else echo 'display: block'; ?>">
		<p><strong>Step 1 - The Setup</strong> - First, you need to set the interest rate you want to payout and what to apply interest to.</p>
		<table class="schedule-table">
			<thead>
				<tr>
					<th class="col-rate">Interest Rate (%)</th>
					<th class="col-base">Based on</th>
					<th class="col-minimum">Minimum</th>
				</tr>
			</thead>
			<tbody>
				<tr>
					<td class="col-rate">
						<input type="text" name="task[rate]" class="large-input" placeholder="%" size="8" value="<?php echo esc_attr( $settings['rate']['amount'] ); ?>" />
						<p><em>Can not be zero.</em></p>
					</td>
					<td class="col-base">
						<select name="task[base]" class="large-input toggle-description">
							<option value="balance" data-target="point-accumilated-balance">Users daily balance (default)</option>
							<option value="accumulated" data-target="point-accumilated-desc">Amount of <?php echo $mycred->plural(); ?> accumulated each day</option>
						</select>
						<div id="point-accumilated-desc" class="toggle-this" style="display: none;">
							<p><em>Interest is paid based on the amount of points a user has gained during each day. If they have not gained any points, they do not qualify for that day.</em></p>
						</div>
						<div id="point-accumilated-balance" class="toggle-this" style="display: block;">
							<p><em>Interest is paid based on the users balance each day.</em></p>
						</div>
					</td>
					<td class="col-minimum">
						<input type="text" name="task[minimum]" class="large-input" placeholder="<?php echo $mycred->zero(); ?>" size="4" value="" />
						<p><em>Optional minimum amount that is required to qualify. Use zero to disable.</em></p>
					</td>
				</tr>
			</tbody>
		</table>
		<p><strong>Step 2 - Start Date</strong> - The date to start compounding interest from. The date of the first log entry in your database has been pre-populated. It is not possible to use a date earlier then this.</p>
		<table class="schedule-table">
			<thead>
				<tr>
					<th style="width: 40%;">Year</th>
					<th style="width: 30%;">Month</th>
					<th style="width: 30%;">Day</th>
				</tr>
			</thead>
			<tbody>
				<tr>
					<td class="col-year">
						<input type="text" name="task[year]" class="large-input" placeholder="YYYY" maxlength="4" size="8" value="<?php echo date( 'Y', $first_entry ); ?>" />
					</td>
					<td class="col-month">
						<input type="text" name="task[month]" class="large-input" placeholder="MM" maxlength="2" size="4" value="<?php echo date( 'm', $first_entry ); ?>" />
					</td>
					<td class="col-day">
						<input type="text" name="task[day]" class="large-input" placeholder="DD" maxlength="2" size="4" value="<?php echo date( 'd', $first_entry ); ?>" />
					</td>
			</tbody>
		</table>
<?php

		$excluded = $settings['exclude_roles'];
		if ( ! empty( $excluded ) )
			echo '<p><strong>Warning</strong><br /><em>Based on your saved settings, the following roles will be excluded: <code>' . implode( '</code>, <code>', $excluded ) . '</code>. If this is not to your liking, please click "Cancel" and change your settings first.</em></p>';

?>
		<p><input type="submit" class="button button-primary" value="Continue" /></p>
	</div>
</div>
<?php

		$content = ob_get_contents();
		ob_end_clean();

		return $content;

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'get_bank_task_runner' ) ) :
	function get_bank_task_runner( $point_type = 'mycred_default', $service = '', $item = '', $form ) {

		global $wpdb;

		$mycred      = mycred( $point_type );
		$date_format = get_option( 'date_format' );
		$time_format = get_option( 'time_format' );
		$settings    = get_banking_addon_settings_mycred( $point_type );
		$settings    = $settings['service_prefs'][ $service ];

		$log_entries = mycred_have_log_entries( $point_type );
		if ( $form['year'] == '' || $form['month'] == '' || $form['day'] == '' )
			$first_entry = $wpdb->get_var( $wpdb->prepare( "SELECT time FROM {$mycred->log_table} WHERE ctype = %s ORDER BY id ASC LIMIT 1;", $point_type ) );
		else
			$first_entry = strtotime( $form['year'] . '-' . $form['month'] . '-' . $form['day'] . ' 00:00:00' );

		$days        = ceil( ( time() - $first_entry ) / DAY_IN_SECONDS );
		$users       = get_qualified_users_banking( $point_type, $settings['exclude_ids'], $settings['exclude_roles'], $form['minimum'] );

		ob_start();

		if ( $first_entry === false || $first_entry <= 0 )
			echo '<div class="alert alert-warning">No users qualify based on your setup. Check your exclusion settings both for the banking add-on and for your point type.</div>';

		if ( $users === 0 )
			echo '<div class="alert alert-warning">No users qualify based on your setup. Check your exclusion settings both for the banking add-on and for your point type.</div>';

?>
<div id="task-warning">
	<div class="padded-form-body">
		<p><strong>Step 2 - Ready to Run</strong> Once you click "Start" the task will begin to compound interest one day at a time from the start date. DO NOT CLOSE this modal window until finished!</p>
		<table class="schedule-table bordered-table">
			<thead>
				<tr>
					<th class="col-">Point Type</th>
					<?php if ( $form['base'] == 'accumulated' ) : ?><th class="col-">Log Entries</th><?php endif; ?>
					<th class="col-">Start Date</th>
					<th class="col-">No. of Days</th>
					<th class="col-">No. of Users</th>
				</tr>
			</thead>
			<tbody>
				<tr>
					<td class="col-"><?php echo $mycred->plural(); ?></td>
					<?php if ( $form['base'] == 'accumulated' ) : ?><td class="col-"><?php echo esc_attr( $log_entries ); ?></td><?php endif; ?>
					<td class="col-"><?php echo date_i18n( $date_format, $first_entry ); ?></td>
					<td class="col-"><?php echo $days; ?></td>
					<td class="col-"><?php echo $users; ?></td>
				</tr>
			</tbody>
		</table>
		<p><button type="button" class="button button-primary slide-content"<?php if ( $users === 0 ) echo ' disabled="disabled"'; ?> id="start-running-the-task-button" data-up="task-warning" data-down="task-<?php echo esc_attr( $point_type ); ?>-<?php echo esc_attr( $service ); ?>">Start</button></p>
	</div>
</div>
<div id="task-<?php echo esc_attr( $point_type ); ?>-<?php echo esc_attr( $service ); ?>" style="display: none;">
	<div id="task-process-indicator"><div class="loading-indicator"></div></div>
	<div class="padded-form-body">
		<table class="schedule-table bordered-table">
			<thead>
				<tr>
					<th class="col-status">Status</th>
					<th class="col-date">Date</th>
					<th class="col-rate">Rate</th>
					<th class="col-interest">Compounded</th>
					<th class="col-users">Updates</th>
				</tr>
			</thead>
			<tbody>
				<tr>
					<td class="col-status" id="task-current-status">Running</td>
					<td class="col-date" id="task-current-date"><?php echo date_i18n( $date_format, $first_entry ); ?></td>
					<td class="col-rate"><?php echo $form['rate']; ?> %</td>
					<td class="col-interest" id="task-current-interest"><?php echo $mycred->before; ?><span style="padding: 0 12px;"><?php echo $mycred->zero(); ?></span><?php echo $mycred->after; ?></td>
					<td class="col-users" id="task-current-users">0</td>
				</tr>
			</tbody>
		</table>
		<p>Remember to either delete or payout any calculated interest before running this tool again or it will duplicate values.</p>
		<p>
			<button type="button" id="stop-current-running-task" class="button button-secondary">Stop Task</button>
			<button type="button" class="button button-primary slide-content" style="display: none;" id="payout-compound-button">Payout Compounded Interest</button>
		</p>
	</div>
</div>
<script type="text/javascript">
jQuery(function($) {

	var run_pointtype    = '<?php echo $point_type; ?>';
	var run_basedon      = '<?php echo $form['base']; ?>';
	var run_day          = <?php echo $first_entry; ?>;

	var task_status_el   = $( '#task-current-status' );
	var task_date_el     = $( '#task-current-date' );
	var task_interest_el = $( '#task-current-interest span' );
	var task_users_el    = $( '#task-current-users' );
	var cancel_button_el = $( '#stop-current-running-task' );

	var results_interest = 0;
	var results_users    = 0;
	var results_status   = $( '#task-process-indicator' );
	var run_task         = true;
	var decomals         = <?php echo ( $mycred->format['decimals'] + 2 ); ?>;

	function stopped_task_process( status, message ) {

		cancel_button_el.attr( 'disabled', 'disabled' );
		results_status.hide();
		run_task = false;
		task_status_el.empty().text( status );
		task_date_el.empty().text( message );

	}

	/**
	 * 
	 * @since 1.0
	 * @version 1.0
	 */
	function run_banking_interest_task( index ) {

		if ( run_task === false ) return false;

		$.ajax({
			type        : "POST",
			data        : {
				action : 'run-mycred-bank-task',
				_nonce  : '<?php echo wp_create_nonce( 'run-mycred-bank-task' ); ?>',
				day     : index,
				type    : run_pointtype,
				base    : run_basedon,
				min     : '<?php if ( $form['minimum'] == '' ) echo '0'; else echo $form['minimum']; ?>',
				rate    : '<?php echo $form['rate']; ?>'
			},
			dataType    : "JSON",
			url         : '<?php echo admin_url( 'admin-ajax.php' ); ?>',
			success     : function( response ) {

				if ( response.success === undefined ) {
					stopped_task_process( 'Stopped', 'Expired nonce. Please refresh this page.' );
					return false;
				}

				else {

					console.log( 'Interest: ' + response.data.compounded );

					results_interest = parseFloat( results_interest );
					results_interest += parseFloat( response.data.compounded );
					task_interest_el.empty().text( parseFloat( results_interest ).toFixed(decomals) );

					var usersdone = parseInt( response.data.users );
					var newusers  = results_users + usersdone;
					results_users = parseInt( newusers );
					task_users_el.empty().text( results_users );

					// Error
					if ( ! response.success ) {
						stopped_task_process( 'Error', response.data );
						return false;
					}

					else {

						task_date_el.empty().text( response.data.next );

						if ( response.data.finished ) {
							stopped_task_process( 'Finished', response.data.message );
							if ( results_interest != 0 ) {
								$( '#payout-compound-button' ).fadeIn();
							}
							return false;
						}

						else {

							return run_banking_interest_task( response.data.index );

						}

					}

				}

			},
			error       : function( jqXHR, textStatus, errorThrown ) {

				console.log( jqXHR + ':' + textStatus + ':' + errorThrown );
				stopped_task_process( 'Stopped', textStatus );
				return false;

			}
		});

	}

	/**
	 * 
	 * @since 1.0
	 * @version 1.0
	 */
	$( '#start-running-the-task-button' ).click(function(){

		console.log( 'start task' );

		if ( run_task === false ) return false;

		cancel_button_el.removeAttr( 'disabled' );
		run_banking_interest_task( run_day );

	});

	/**
	 * 
	 * @since 1.0
	 * @version 1.0
	 */
	$( '#stop-current-running-task' ).click(function(){

		console.log( 'stop task' );

		stopped_task_process( 'Stopped', 'Process aborted by you.' );

	});

	/**
	 * 
	 * @since 1.0
	 * @version 1.0
	 */
	$( '#payout-compound-button' ).click(function(){

		$( '#run-task-form' ).empty().append( '<input type="hidden" name="action" id="run-task-field-action" value="payout" /><input type="hidden" name="service" id="run-task-field-service" value="interest" /><input type="hidden" name="item" id="run-task-field-item" value="compound" /><input type="hidden" name="point_type" id="run-task-field-point-type" value="' + run_pointtype + '" />' ).submit();

	});

});
</script>
<?php

		$content = ob_get_contents();
		ob_end_clean();

		return $content;

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'get_bank_task_payout_form' ) ) :
	function get_bank_task_payout_form( $point_type, $service, $item, $users ) {

		global $wpdb;

		$mycred   = mycred( $point_type );
		$settings = get_banking_addon_settings_mycred( $point_type );
		$settings = $settings['service_prefs'][ $service ];

		$total = $wpdb->get_var( $wpdb->prepare( "SELECT SUM( meta_value ) FROM {$wpdb->usermeta} WHERE meta_key = %s;", $point_type . '_comp' ) );

		ob_start();

?>
<div id="task-payout-indicator" style="display: none;"><div class="loading-indicator"></div></div>
<div class="padded-form-body">
	<p><em>The compounded interest shown below is pending payout and have not yet been applied to users accounts.</em></p>
	<table class="schedule-table bordered-table">
		<thead>
			<tr>
				<th class="col-">Point Type</th>
				<th class="col-">No. of Users</th>
				<th class="col-">Total Payable</th>
				<th class="col-">Completed</th>
			</tr>
		</thead>
		<tbody>
			<tr>
				<td class="col-"><?php echo $mycred->plural(); ?></td>
				<td class="col-"><?php echo $users; ?></td>
				<td class="col-"><?php echo $mycred->format_creds( $total ); ?></td>
				<td class="col-" id="number-of-users-completed">0</td>
			</tr>
		</tbody>
	</table>
	<p><button type="button" class="button button-primary" id="start-payout-interest-button">Start Payout</button> <button type="button" id="stop-current-running-task" class="button button-secondary" style="display: none;">Stop Task</button></p>
</div>
<script type="text/javascript">
jQuery(function($) {

	var run_pointtype     = '<?php echo $point_type; ?>';
	var task_users_run_el = $( '#number-of-users-completed' );
	var cancel_button_el  = $( '#stop-current-running-task' );

	var users_per_cycle = <?php if ( $users > 1000 ) echo 50; else echo 25; ?>;
	var users_run       = 0;
	var run_task        = true;

	function stopped_task_process( message ) {

		cancel_button_el.hide();
		$( '#start-payout-interest-button' ).removeAttr( 'disabled' );
		run_task = false;
		alert( message );

	}

	/**
	 * 
	 * @since 1.0
	 * @version 1.0
	 */
	function run_banking_interest_task() {

		if ( run_task === false ) return false;

		$.ajax({
			type        : "POST",
			data        : {
				action : 'payout-compounded-interest',
				_nonce  : '<?php echo wp_create_nonce( 'payout-compound-interest' ); ?>',
				number  : users_per_cycle,
				type    : run_pointtype
			},
			dataType    : "JSON",
			url         : '<?php echo admin_url( 'admin-ajax.php' ); ?>',
			beforeSend  : function() {

				console.log( 'New ajax request' );

			},
			success     : function( response ) {

				if ( response.success === undefined ) {
					stopped_task_process( 'Expired nonce. Please refresh this page.' );
					return false;
				}

				else {

					var usersdone = parseInt( response.data.users );
					var newusers  = users_run + usersdone;
					users_run     = parseInt( newusers );
					task_users_run_el.empty().text( users_run );

					// Error
					if ( ! response.success ) {
						stopped_task_process( response.data );
						return false;
					}

					else {

						if ( response.data.finished ) {
							stopped_task_process( response.data.message );
							$( '#start-payout-interest-button' ).remove();
							return false;
						}

						else {

							return run_banking_interest_task();

						}

					}

				}

			},
			error       : function( jqXHR, textStatus, errorThrown ) {

				console.log( jqXHR + ':' + textStatus + ':' + errorThrown );
				stopped_task_process( textStatus );
				return false;

			}
		});

	}

	/**
	 * 
	 * @since 1.0
	 * @version 1.0
	 */
	$( '#start-payout-interest-button' ).click(function(e){

		e.preventDefault();

		$(this).attr( 'disabled', 'disabled' );
		cancel_button_el.show();

		run_banking_interest_task( 0 );

	});

});
</script>
<?php

		$content = ob_get_contents();
		ob_end_clean();

		return $content;

	}
endif;
