<?php
if ( ! defined( 'MYCRED_DEBUG_BANKING_VER' ) ) exit;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'mycred_debug_banking_screen_central' ) ) :
	function mycred_debug_banking_screen_central( $is_over_seven = false, $version = NULL ) {

		$point_types    = mycred_get_types();

?>
<h3>Central Banking Service</h3>
<table class="wp-list-table widefat fixed striped users">
	<thead>
		<tr>
			<th style="width: 10%;">Point Type</th>
			<th style="width: 15%;">Service Status</th>
			<th style="width: 25%;">The Central Bank User</th>
			<th style="width: 10%;">Balance</th>
			<th style="width: 40%;">Issues</th>
		</tr>
	</thead>
	<tbody id="the-list">
<?php

		foreach ( $point_types as $type_id => $label ) {

			$issues       = array();
			$central_bank = $account_balance = '-';

			$mycred       = mycred( $type_id );
			$settings     = get_banking_addon_settings_mycred( $type_id );
			$enabled      = ( ( in_array( 'central', $settings['active'] ) ) ? true : false );
			$settings     = $settings['service_prefs']['central'];

			if ( $enabled ) {

				$bank_user_id = $settings['bank_id'];
				$bank_user    = get_userdata( $bank_user_id );
				if ( ! isset( $bank_user->ID ) )
					$issues[] = 'The user nominated to represent the central bank account does not exist or the settings are empty / invalid.';

				else {

					$central_bank = $bank_user->display_name . ' <small>ID: ' . $bank_user_id . '</small>';
					if ( $mycred->exclude_user( $bank_user_id ) )
						$issues[] = 'The central bank account is excluded from using this point type! Please nominate a different user or disable exclusion for this account on the Settings page under "Core Settings".';

					else {

						$balance = $mycred->get_users_balance( $bank_user_id, $type_id );
						if ( $balance <= $mycred->zero() ) {
							$issues[] = $mycred->template_tags_general( 'The central bank has no %_plural%. No %_plural% will be paid out until the account balance increases.' );
							$account_balance = '<span class="text-red">' . $mycred->format_creds( $balance ) . '</span>';
						}
						else {
							$account_balance = '<span class="text-green">' . $mycred->format_creds( $balance ) . '</span>';
						}

					}

				}

			}

?>
		<tr>
			<td><?php echo $mycred->plural(); ?></td>
			<td><?php if ( $enabled ) echo '<span class="text-green">Enabled</span>'; else echo '<span class="text-red">Disabled</span>'; ?></td>
			<td><?php echo $central_bank; ?></td>
			<td><?php echo $account_balance; ?></td>
			<td>
<?php

			if ( ! empty( $issues ) ) {
				echo '<ol><li>' . implode( '</li><li>', $issues ) . '</li></ul>';
			}
			else {
				echo '<p>No issues found.</p>';
			}

?>
			</td>
		</tr>
		<tr>
			<td colspan="5"><em>No actions available.</em></td>
		</tr>
<?php

		}

?>
	</tbody>
</table>
<?php

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'mycred_debug_banking_screen_compound' ) ) :
	function mycred_debug_banking_screen_compound( $is_over_seven = false, $version = NULL ) {

		$point_types    = mycred_get_types();
		$date_format    = get_option( 'date_format' );
		$time_format    = get_option( 'time_format' );

?>
<h3>Compound Interest Service</h3>
<table class="wp-list-table widefat fixed striped users">
	<thead>
		<tr>
			<th style="width: 10%;">Point Type</th>
			<th style="width: 10%;">Service Status</th>
			<th style="width: 20%;">Compound CRON</th>
			<th style="width: 20%;">Payout CRON</th>
			<th style="width: 40%;">Issues</th>
		</tr>
	</thead>
	<tbody id="the-list">
<?php

		foreach ( $point_types as $type_id => $label ) {

			$issues       = array();
			$actions      = array();
			$compound     = $payout = false;

			$mycred       = mycred( $type_id );
			$settings     = get_banking_addon_settings_mycred( $type_id );
			$enabled      = ( ( in_array( 'interest', $settings['active'] ) ) ? true : false );
			$settings     = $settings['service_prefs']['interest'];

			if ( $enabled ) {

				if ( $settings['rate']['amount'] == 0 )
					$issues[] = 'You have set to payout zero % in interest rate.';

				if ( strlen( $settings['log'] ) == 0 )
					$issues[] = 'The "Log Template" seems to be empty. Points will not be paid out.';

				if ( $settings['min_balance'] > 0 ) {

					$balance_count = get_users_with_mycred_balance( $settings['min_balance'], $type_id, $mycred );
					if ( $balance_count <= 0 )
						$issues[] = 'Based on the minimum balance requirement you set, right now, there are no users that qualify for interest.';

				}

				$option_id = 'mycred_bank_interest_comp' . $type_id;
				$compound  = wp_next_scheduled( $option_id );
				if ( $compound === false ) {
					$issues[]  = 'Compound Interest rate CRON is not scheduled!';
					$actions[] = '<button type="button" class="button button-secondary schedule-cron-button" data-action="reschedule" data-service="interest" data-item="compound" id="reschedule-interest-compound-' . $type_id . '" data-type="' . $type_id . '">Schedule Compound Cron</button>';
				}

				$option_id = 'mycred_bank_interest_pay' . $type_id;
				$payout    = wp_next_scheduled( $option_id );
				if ( $payout === false ) {
					$issues[]  = 'Interest Payout CRON is not scheduled!';
					$actions[] = '<button type="button" class="button button-secondary schedule-cron-button" data-action="reschedule" data-service="interest" data-item="payout" id="reschedule-interest-payout-' . $type_id . '" data-type="' . $type_id . '">Schedule Payout Cron</button>';
				}

				if ( mycred_have_log_entries( $type_id ) )
					$actions[] = '<button type="button" class="button button-primary run-task-button" data-action="run" data-service="interest" data-item="compound" id="run-interest-compound-' . $type_id . '" data-type="' . $type_id . '">Apply Interest</button>';

				if ( mycred_have_compounded_interest( $type_id ) ) {
					$actions[] = '<button type="button" class="button button-primary run-task-button" data-action="payout" data-service="interest" data-item="compound" id="run-interest-compound-payout-' . $type_id . '" data-type="' . $type_id . '">Payout Interest</button>';
					$actions[] = '<button type="button" class="button button-primary run-task-button" data-action="delete" data-service="interest" data-item="compound" id="delete-interest-compound-payout-' . $type_id . '" data-type="' . $type_id . '">Delete Pending Payouts</button>';
				}

			}

?>
		<tr>
			<td><?php echo $mycred->plural(); ?></td>
			<td><?php if ( $enabled ) echo '<span class="text-green">Enabled</span>'; else echo '<span class="text-red">Disabled</span>'; ?></td>
			<td><?php if ( $compound !== false ) echo date( $date_format . ' @ ' . $time_format, mycred_gmt_time_to_local_time( $compound ) ); else echo '-'; ?></td>
			<td><?php if ( $payout !== false ) echo date( $date_format . ' @ ' . $time_format, mycred_gmt_time_to_local_time( $payout ) ); else echo '-'; ?></td>
			<td>
<?php

			if ( ! empty( $issues ) ) {
				echo '<ol><li>' . implode( '</li><li>', $issues ) . '</li></ul>';
			}
			else {
				echo '<p>No issues found.</p>';
			}

?>
			</td>
		</tr>
		<tr>
			<td colspan="5">
<?php

			if ( ! empty( $actions ) ) {

?>
				<p><strong>Actions:</strong></p>
				<?php echo implode( ' ', $actions ); ?>
<?php

			}
			else echo '<em>No actions available.</em>';

?>
			</td>
		</tr>
<?php

		}

?>
	</tbody>
</table>
<div id="scedule-cron-modal" style="display: none;">
	<div class="mycred-container">
		<form class="form" method="post" action="" id="cron-schedule-form">
			<input type="hidden" name="action" id="cron-schedule-field-action" value="" />
			<input type="hidden" name="service" id="cron-schedule-field-service" value="" />
			<input type="hidden" name="item" id="cron-schedule-field-item" value="" />
			<input type="hidden" name="point_type" id="cron-schedule-field-point-type" value="" />
		</form>
		<div id="form-submission-status-schedule" style="display:none;"><div id="mycred-processing"><div class="loading-indicator"></div></div></div>
	</div>
</div>
<div id="run-task-modal" style="display: none;">
	<div class="mycred-container">
		<form class="form" method="post" action="" id="run-task-form">
			<input type="hidden" name="action" id="run-task-field-action" value="" />
			<input type="hidden" name="service" id="run-task-field-service" value="" />
			<input type="hidden" name="item" id="run-task-field-item" value="" />
			<input type="hidden" name="point_type" id="run-task-field-point-type" value="" />
		</form>
		<div id="form-submission-status-task" style="display:none;"><div id="mycred-processing-task"><div class="loading-indicator"></div></div></div>
	</div>
</div>
<?php

	}
endif;

/**
 * 
 * @since 1.0
 * @version 1.0
 */
if ( ! function_exists( 'mycred_debug_banking_screen_recurring' ) ) :
	function mycred_debug_banking_screen_recurring( $is_over_seven = false, $version = NULL ) {

		$point_types    = mycred_get_types();

?>
<h3>Recurring Payouts Service</h3>
<table class="wp-list-table widefat fixed striped users">
	<thead>
		<tr>
			<th style="width: 20%;">Point Type</th>
			<th style="width: 20%;">Service Status</th>
			<th style="width: 60%;">Issues</th>
		</tr>
	</thead>
	<tbody id="the-list">
<?php

		foreach ( $point_types as $type_id => $label ) {

			$issues       = array();
			$central_bank = $account_balance = '-';

			$mycred       = mycred( $type_id );
			$settings     = get_banking_addon_settings_mycred( $type_id );
			$enabled      = ( ( in_array( 'payouts', $settings['active'] ) ) ? true : false );

			if ( $enabled ) {

				

			}

?>
		<tr>
			<td><?php echo $mycred->plural(); ?></td>
			<td><?php if ( $enabled ) echo '<span class="text-green">Enabled</span>'; else echo '<span class="text-red">Disabled</span>'; ?></td>
			<td>
<?php

			if ( ! empty( $issues ) ) {
				echo '<ol><li>' . implode( '</li><li>', $issues ) . '</li></ul>';
			}
			else {
				echo '<p>No issues found.</p>';
			}

?>
			</td>
		</tr>
		<tr>
			<td colspan="3"><em>No actions available.</em></td>
		</tr>
<?php

		}

?>
	</tbody>
</table>
<?php

	}
endif;
