<?php
/**
 * Plugin Name: myCRED Banking Debug
 * Plugin URI: http://mycred.me
 * Description: Debug tool for the Banking add-on.
 * Version: 1.0
 * Tags: debug, banking, addon, mycred
 * Author: Gabriel S Merovingi
 * Author URI: http://www.merovingi.com
 * Author Email: support@mycred.me
 * Text Domain: mycreddebug
 * Requires at least: WP 4.2
 */
if ( ! class_exists( 'myCRED_Banking_Addon_Debug' ) ) :
	final class myCRED_Banking_Addon_Debug {

		// Plugin Version
		public $version             = '1.0';

		// Instnace
		protected static $_instance = NULL;

		// Current session
		public $session             = NULL;

		public $mycred_version      = false;
		public $post_one_seven      = false;

		public $documentation_url   = 'http://mycred.me/add-ons/banking/';
		public $support_url         = 'http://mycred.me/support/';

		/**
		 * Setup Instance
		 * @since 1.7
		 * @version 1.0
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Not allowed
		 * @since 1.0
		 * @version 1.0
		 */
		public function __clone() { _doing_it_wrong( __FUNCTION__, 'Cheatin&#8217; huh?', '1.0' ); }

		/**
		 * Not allowed
		 * @since 1.0
		 * @version 1.0
		 */
		public function __wakeup() { _doing_it_wrong( __FUNCTION__, 'Cheatin&#8217; huh?', '1.0' ); }

		/**
		 * Define
		 * @since 1.7
		 * @version 1.0
		 */
		private function define( $name, $value ) {
			if ( ! defined( $name ) )
				define( $name, $value );
		}

		/**
		 * Require File
		 * @since 1.0
		 * @version 1.0
		 */
		public function file( $required_file ) {
			if ( file_exists( $required_file ) )
				require_once $required_file;
			else
				_doing_it_wrong( 'myCRED_Banking_Addon_Debug->file()', 'Requested file ' . $required_file . ' not found.', '1.0' );
		}

		/**
		 * Construct
		 * @since 1.0
		 * @version 1.0
		 */
		public function __construct() {

			$this->define_constants();
			$this->includes();

			add_action( 'plugins_loaded', array( $this, 'wp_plugins_loaded' ) );
			add_action( 'init',           array( $this, 'wp_init' ) );

		}

		/**
		 * Define Constants
		 * First, we start with defining all requires constants if they are not defined already.
		 * @since 1.0
		 * @version 1.0
		 */
		private function define_constants() {

			$this->define( 'MYCRED_DEBUG_BANKING_VER',          $this->version );
			$this->define( 'MYCRED_DEBUG_BANKING',              __FILE__ );
			$this->define( 'MYCRED_DEBUG_BANKING_DIR',          plugin_dir_path( MYCRED_DEBUG_BANKING ) );
			$this->define( 'MYCRED_DEBUG_BANKING_INCLUDES_DIR', MYCRED_DEBUG_BANKING_DIR . 'includes/' );

		}

		/**
		 * Include Plugin Files
		 * @since 1.0
		 * @version 1.0
		 */
		public function includes() {

			$this->file( MYCRED_DEBUG_BANKING_INCLUDES_DIR . 'debug-functions.php' );
			$this->file( MYCRED_DEBUG_BANKING_INCLUDES_DIR . 'debug-screen-functions.php' );
			$this->file( MYCRED_DEBUG_BANKING_INCLUDES_DIR . 'debug-updates.php' );

		}

		/**
		 * Init
		 * @since 1.0
		 * @version 1.0
		 */
		public function wp_plugins_loaded() {

			// No need to do anything if myCRED is disabled
			if ( ! function_exists( 'mycred' ) ) return;

			$this->mycred_version = myCRED_VERSION;

			if ( version_compare( $this->mycred_version, '1.7', '>=' ) )
				$this->post_one_seven = true;

		}

		/**
		 * Init
		 * @since 1.0
		 * @version 1.0
		 */
		public function wp_init() {

			// No need to do anything if myCRED or the add-on are disabled
			if ( $this->mycred_version === false || ! class_exists( 'myCRED_Banking_Module' ) ) return;

			add_action( 'admin_menu', array( $this, 'add_menu' ) );

			add_action( 'wp_ajax_debug-mycred-bank-schedules', array( $this, 'ajax_handler_schedules' ) );
			add_action( 'wp_ajax_debug-mycred-bank-tasks',     array( $this, 'ajax_handler_tasks' ) );
			add_action( 'wp_ajax_run-mycred-bank-task',        array( $this, 'ajax_handler_run_task' ) );
			add_action( 'wp_ajax_payout-compounded-interest',  array( $this, 'ajax_handler_payout_interest' ) );

		}

		/**
		 * Add Admin menu
		 * @since 1.0
		 * @version 1.0
		 */
		public function add_menu() {

			$page = add_management_page( __( 'Debug Banking Add-on', 'mycreddebug' ), __( 'Debug Banking', 'mycreddebug' ), 'edit_users', 'mycred-debug-banking', array( $this, 'debug_screen' ) );

			add_action( 'admin_print_styles-' . $page, array( $this, 'debug_screen_header' ) );

		}

		/**
		 * Debug Screen Header
		 * @since 1.0
		 * @version 1.0
		 */
		public function debug_screen_header() {

			wp_enqueue_style( 'mycred-debug-banking', plugins_url( 'assets/css/debug-screen.css', MYCRED_DEBUG_BANKING ), array(), $this->version, 'all' );

			wp_register_script( 'mycred-debug-banking', plugins_url( 'assets/js/debug-screen.js', MYCRED_DEBUG_BANKING ), array( 'jquery', 'jquery-ui-core', 'jquery-ui-dialog', 'jquery-effects-core', 'jquery-effects-slide', 'jquery-numerator' ), $this->version );

			wp_localize_script(
				'mycred-debug-banking',
				'DebugBanking',
				array(
					'ajaxurl'       => admin_url( 'admin-ajax.php' ),
					'schedule'      => wp_create_nonce( 'schedule-bank-cron' ),
					'task'          => wp_create_nonce( 'run-bank-task' ),
					'title'         => esc_attr__( 'Schedule Cron', 'mycred' ),
					'tasktitle'     => esc_attr__( 'Run Banking Task', 'mycred' ),
					'close'         => esc_attr__( 'Close', 'mycred' ),
					'working'       => esc_attr__( 'Processing...', 'mycred' ),
					'schedulefield' => '<input type="hidden" name="action" id="cron-schedule-field-action" value="" /><input type="hidden" name="service" id="cron-schedule-field-service" value="" /><input type="hidden" name="item" id="cron-schedule-field-item" value="" /><input type="hidden" name="point_type" id="cron-schedule-field-point-type" value="" />',
					'taskfields'    => '<input type="hidden" name="action" id="run-task-field-action" value="" /><input type="hidden" name="service" id="run-task-field-service" value="" /><input type="hidden" name="item" id="run-task-field-item" value="" /><input type="hidden" name="point_type" id="run-task-field-point-type" value="" />'
				)
			);
			wp_enqueue_script( 'mycred-debug-banking' );

		}

		/**
		 * Debug Screen
		 * @since 1.0
		 * @version 1.0
		 */
		public function debug_screen() {

			global $wpdb;

?>
<div class="wrap mycred-metabox" id="debug-screen">
	<h1><?php _e( 'Debug Banking Add-on', 'mycreddebug' ); ?> <a href="<?php echo esc_url( $this->documentation_url ); ?>" class="page-title-action" target="_blank">Documentation</a> <a href="<?php echo esc_url( $this->support_url ); ?>" class="page-title-action" target="_blank">Support</a></h1>
	<p>You can find a complete overview of your Banking add-ons setup.</p>
	<table class="wp-list-table widefat fixed striped users">
		<thead>
			<tr>
				<th style="width: 25%;">Website</th>
				<th style="width: 10%;">Multisite</th>
				<th style="width: 10%;">Memory</th>
				<th style="width: 15%;">WordPress</th>
				<th style="width: 15%;">myCRED</th>
				<th style="width: 15%;">PHP</th>
				<th style="width: 15%;">SQL</th>
			</tr>
		</thead>
		<tbody id="the-list">
			<tr>
				<td><?php echo esc_attr( home_url() ); ?></td>
				<td><?php if ( is_multisite() ) echo 'Yes'; else echo 'No'; ?></td>
				<td><?php echo WP_MEMORY_LIMIT; ?></td>
				<td><?php echo esc_attr( $GLOBALS['wp_version'] ); ?></td>
				<td><?php echo esc_attr( $this->mycred_version ); ?></td>
				<td><?php echo phpversion(); ?></td>
				<td><?php echo $wpdb->db_version(); ?></td>
			</tr>
		</tbody>
	</table>
	<?php mycred_debug_banking_screen_central( $this->post_one_seven, $this->mycred_version ); ?>
	<?php mycred_debug_banking_screen_compound( $this->post_one_seven, $this->mycred_version ); ?>
	<?php mycred_debug_banking_screen_recurring( $this->post_one_seven, $this->mycred_version ); ?>
</div>
<?php

		}

		/**
		 * 
		 * @since 1.0
		 * @version 1.0
		 */
		public function ajax_handler_schedules() {

			check_ajax_referer( 'schedule-bank-cron', '_nonce' );

			parse_str( $_POST['form'], $form );

			extract( shortcode_atts( array(
				'action'     => '',
				'service'    => '',
				'item'       => '',
				'point_type' => 'mycred_default'
			), $form ) );

			if ( $action == 'reschedule' ) {

				if ( ! array_key_exists( 'time', $form ) )
					wp_send_json_success( get_bank_cron_schedule_form( $point_type, $service, $item ) );

				$year   = sanitize_text_field( $form['time']['year'] );
				$month  = zeroise( sanitize_text_field( $form['time']['month'] ), 2 );
				$day    = zeroise( sanitize_text_field( $form['time']['day'] ), 2 );
				$hour   = zeroise( sanitize_text_field( $form['time']['hour'] ), 2 );
				$minute = zeroise( sanitize_text_field( $form['time']['minute'] ), 2 );

				$unixtimestamp = mycred_date_to_gmt_time( $year . '-' . $month . '-' . $day . ' ' . $hour . ':' . $minute . ':00' );
				if ( $unixtimestamp === false || $unixtimestamp <= time() ) {
					$content = '<div class="alert alert-warning">Invalid date or time. Please try again.</div>';
					$content .= get_bank_cron_schedule_form( $point_type, $service, $item );
					wp_send_json_success( $content );
				}

				$id     = 'mycred_bank_interest_comp';
				$text   = 'Compound Schedule';
				$button = 'reschedule-interest-compound-' . $point_type;
				if ( $item == 'payout' ) {
					$id     = 'mycred_bank_interest_pay';
					$text   = 'Payout Schedule';
					$button = 'reschedule-interest-payout-' . $point_type;
				}

				wp_schedule_single_event( $unixtimestamp, $id . $point_type );

				$content = '<div class="alert alert-success">' . $text. ' set to: ' . date( get_option( 'date_format' ) . ' @ ' . get_option( 'time_format' ), mycred_gmt_time_to_local_time( $unixtimestamp ) ) . '</div>' . "<script type=\"text/javascript\">jQuery(function($) { $( 'button#{$button}' ).remove(); });</script>";
				wp_send_json_success( $content );

			}

			wp_send_json_error( '
			<input type="hidden" name="action" id="cron-schedule-field-action" value="" />
			<input type="hidden" name="service" id="cron-schedule-field-service" value="" />
			<input type="hidden" name="item" id="cron-schedule-field-item" value="" />
			<input type="hidden" name="point_type" id="cron-schedule-field-point-type" value="" />
			<div class="alert alert-warning">No action</div>' );

		}

		/**
		 * 
		 * @since 1.0
		 * @version 1.0
		 */
		public function ajax_handler_tasks() {

			check_ajax_referer( 'run-bank-task', '_nonce' );

			parse_str( $_POST['form'], $form );

			extract( shortcode_atts( array(
				'action'     => '',
				'service'    => '',
				'item'       => '',
				'point_type' => 'mycred_default'
			), $form ) );

			if ( $action == 'run' ) {

				if ( $item == 'compound' ) {

					if ( mycred_have_log_entries( $point_type ) === false )
						wp_send_json_success( '<div class="alert alert-warning">Your log is empty. We can not run a retroactive task if there are no records.</div>' );

					if ( ! array_key_exists( 'task', $form ) )
						wp_send_json_success( get_bank_task_run_form( $point_type, $service, $item ) );

					$rate = sanitize_text_field( $form['task']['rate'] );
					$rate = str_replace( ',', '.', $rate );
					$rate = str_replace( array( ' ', ',' ), '', $rate );
					if ( $rate == 0 || $rate == '' ) {
						$content  = '<div class="alert alert-warning">The interest rate can not be zero.</div>';
						$content .= get_bank_task_run_form( $point_type, $service, $item, true );
						wp_send_json_success( $content );
					}
					elseif ( $rate < 0 ) {
						$content  = '<div class="alert alert-warning">The interest rate can not be negative.</div>';
						$content .= get_bank_task_run_form( $point_type, $service, $item, true );
						wp_send_json_success( $content );
					}

					$start_from = strtotime( $form['task']['year'] . '-' . $form['task']['month'] . '-' . $form['task']['day'] . ' 00:00:00' );
					if ( $start_from === false || $start_from <= 0 ) {
						$content  = '<div class="alert alert-warning">Invalid start date. Please try again.</div>';
						$content .= get_bank_task_run_form( $point_type, $service, $item, true );
						wp_send_json_success( $content );
					}

					wp_send_json_success( get_bank_task_runner( $point_type, $service, $item, $form['task'] ) );

				}

			}

			elseif ( $action == 'payout' ) {

				$compound = mycred_have_compounded_interest( $point_type );
				if ( $compound === false )
					wp_send_json_success( '<div class="alert alert-warning">There is no compounded interest to be paid out.</div>' );

				wp_send_json_success( get_bank_task_payout_form( $point_type, $service, $item, $compound ) );

			}

			elseif ( $action == 'delete' ) {

				$compound = mycred_have_compounded_interest( $point_type );
				if ( $compound === false )
					wp_send_json_success( '<div class="alert alert-warning">There is no compounded interest to delete.</div>' );

				if ( array_key_exists( 'token', $form ) && wp_verify_nonce( $form['token'], 'delete-compounded-interest' ) ) {

					global $wpdb;

					$wpdb->delete(
						$wpdb->usermeta,
						array( 'meta_key' => $point_type . '_comp' ),
						array( '%s' )
					);

					wp_send_json_success( '<div class="alert alert-success">All compounded interests have been deleted.</div>' );

				}

				wp_send_json_success( '
			<input type="hidden" name="action" id="run-task-field-action" value="delete" />
			<input type="hidden" name="service" id="run-task-field-service" value="interest" />
			<input type="hidden" name="item" id="run-task-field-item" value="compound" />
			<input type="hidden" name="point_type" id="run-task-field-point-type" value="' . $point_type . '" />
			<input type="hidden" name="token" value="' . wp_create_nonce( 'delete-compounded-interest' ) . '" />
			<div class="alert alert-warning">Are you sure you want to delete compounded interests for this point type?</div>
			<div class="padded-form-body"><input type="submit" class="button button-primary" id="confirm-compound-deletion" value="Yes - Delete All" /></div>' . "<script type=\"text/javascript\">jQuery(function($) { $( '#confirm-compound-deletion' ).click(function(){ $( '#delete-interest-compound-payout-{$point_type}' ).remove(); $( '#run-interest-compound-payout-{$point_type}' ).remove(); }); });</script>" );

			}

			wp_send_json_error( '
			<input type="hidden" name="action" id="run-task-field-action" value="" />
			<input type="hidden" name="service" id="run-task-field-service" value="" />
			<input type="hidden" name="item" id="run-task-field-item" value="" />
			<input type="hidden" name="point_type" id="run-task-field-point-type" value="" />
			<div class="alert alert-warning">No action</div>' );

		}

		/**
		 * 
		 * @since 1.0
		 * @version 1.0
		 */
		public function ajax_handler_run_task() {

			check_ajax_referer( 'run-mycred-bank-task', '_nonce' );

			global $wpdb;

			$excludes_list = '';
			$compounded    = 0;
			$users         = 0;

			$unixtimestamp = absint( $_POST['day'] );
			$minimum       = sanitize_text_field( $_POST['min'] );
			$basedon       = sanitize_text_field( $_POST['base'] );
			$rate          = sanitize_text_field( $_POST['rate'] );
			$point_type    = sanitize_text_field( $_POST['type'] );

			$mycred        = mycred( $point_type );
			if ( $minimum == '' ) $minimum = 0;
			$minimum       = $mycred->number( $minimum );

			$day_start     = strtotime( date( 'Y-m-d 00:00:00', $unixtimestamp ) );
			if ( $day_start <= 0 )
				wp_send_json_error( 'Bad day' );

			$next_day      = $day_start + DAY_IN_SECONDS;
			$day_end       = $next_day - 1;

			$settings      = get_banking_addon_settings_mycred( $point_type );
			$settings      = $settings['service_prefs']['interest'];

			$excluded      = get_qualified_users_banking( $point_type, $settings['exclude_ids'], $settings['exclude_roles'], $minimum, false );

			if ( ! empty( $excluded ) )
				$excludes_list = " AND user_id NOT IN (" . implode( ', ', $excluded ) . ")";

			$todays_query = false;
			if ( $basedon === 'accumulated' ) {

				$todays_query = $wpdb->get_results( $wpdb->prepare( "SELECT user_id, SUM( creds ) as total FROM {$mycred->log_table} WHERE ctype = %s {$excludes_list} AND time BETWEEN %d AND %d GROUP BY user_id ORDER BY user_id ASC;", $point_type, $day_start, $day_end ) );

				if ( $todays_query !== false ) {

					if ( ! empty( $todays_query ) ) {

						foreach ( $todays_query as $entry ) {

							$total = (double) $entry->total;
							if ( $entry->total <= $mycred->zero() ) continue;

							if ( $total < $minimum ) continue;

							$past  = mycred_get_user_meta( $entry->user_id, $point_type . '_comp' );
							if ( $past == '' ) $past = 0;

							$interest   = $total * ( (double) $rate / 100 );
							$past       = (double) $past + (double) $interest;
							$compounded = (double) $compounded + (double) $interest;
							mycred_update_user_meta( $entry->user_id, $point_type . '_comp', '', $past );

							$users++;

						}

					}

				}
				else {
					wp_send_json_error( 'Unknown based on option.' );
				}

			}

			elseif ( $basedon === 'balance' ) {

				$period       = 1;
				$days_in_year = 365;
				if ( date( 'L' ) )
					$days_in_year = 366;

				$period = ( $period / $days_in_year );

				$todays_query = $wpdb->get_results( $wpdb->prepare( "SELECT user_id, SUM( creds ) as total FROM {$mycred->log_table} WHERE ctype = %s {$excludes_list} AND time <= %d GROUP BY user_id ORDER BY user_id ASC;", $point_type, $day_end ) );

				if ( $todays_query !== false ) {

					if ( ! empty( $todays_query ) ) {

						foreach ( $todays_query as $entry ) {

							$total = (double) $entry->total;
							if ( $entry->total <= $mycred->zero() ) continue;

							if ( $total < $minimum ) continue;

							$saved_interest  = mycred_get_user_meta( $entry->user_id, $point_type . '_comp' );
							if ( $saved_interest == '' ) $saved_interest = 0;

							$interest_rate = $rate / 100;
							$interest      = ( ( $entry->total + $saved_interest ) * $interest_rate ) * $period;
							$compounded   += $interest;

							mycred_update_user_meta( $entry->user_id, $point_type . '_comp', '', $interest );

							$users++;

						}

					}

				}
				else {
					wp_send_json_error( 'Unknown based on option.' );
				}

			}

			$finished = false;
			if ( $next_day > current_time( 'timestamp' ) )
				$finished = true;

			// Success and continue
			wp_send_json_success( array(
				'finished'   => $finished,
				'next'       => date( get_option( 'date_format' ), $next_day ),
				'index'      => $next_day,
				'compounded' => $compounded,
				'users'      => $users,
				'message'    => 'Finished up until today.'
			) );

		}

		/**
		 * 
		 * @since 1.0
		 * @version 1.0
		 */
		public function ajax_handler_payout_interest() {

			check_ajax_referer( 'payout-compound-interest', '_nonce' );

			global $wpdb;

			$number     = absint( $_POST['number'] );
			$point_type = sanitize_key( $_POST['type'] );
			$mycred     = mycred( $point_type );
			$meta_key   = $point_type . '_comp';

			$completed  = 0;
			$settings   = get_banking_addon_settings_mycred( $point_type );
			$settings   = $settings['service_prefs']['interest'];

			$compounded = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$wpdb->usermeta} WHERE meta_key = %s;", $meta_key ) );

			if ( ! empty( $compounded ) ) {

				foreach ( $compounded as $entry ) {

					if ( ! $mycred->has_entry( 'interest', $entry->umeta_id, $entry->user_id, '', $point_type ) ) {

						$interest = $mycred->number( $entry->meta_value );
						$payout   = $mycred->add_creds(
							'interest',
							$entry->user_id,
							$interest,
							$settings['log'],
							$entry->umeta_id,
							'',
							$point_type
						);

						if ( $payout ) {

							$completed++;
							delete_user_meta( $entry->user_id, $meta_key );

						}
						$completed++;

					}

				}

			}

			$finished = false;
			if ( $completed < $number )
				$finished = true;

			// Success and continue
			wp_send_json_success( array(
				'finished'   => $finished,
				'users'      => $completed,
				'message'    => 'Everyone has been paid.'
			) );

		}

	}
endif;

function mycred_debug_banking() {
	return myCRED_Banking_Addon_Debug::instance();
}
mycred_debug_banking();
